@@ -184,10 +184,12 @@ let collect_formatters buf pps f =
     raise exn
 
 let discard_formatters pps f =
+  match pps with [] -> f () | fpp :: pps_rest ->
   (* First flush all formatters. *)
   List.iter (fun pp -> Format.pp_print_flush pp ()) pps;
   (* Save all formatter functions. *)
-  let save = List.map (fun pp -> Format.pp_get_formatter_out_functions pp ()) pps in
+  let h = Format.pp_get_formatter_out_functions fpp () in
+  let save = h :: List.map (fun pp -> Format.pp_get_formatter_out_functions pp ()) pps_rest in
   let restore () =
     List.iter2
       (fun pp out_functions ->
@@ -197,9 +199,10 @@ let discard_formatters pps f =
   in
   (* Output functions. *)
   let out_functions = {
+      h with (* OCaml 5.4 added a new field *)
     Format.out_string = (fun _ _ _ -> ()); out_flush = ignore;
-    out_newline = ignore; out_spaces = ignore ; out_indent = ignore
-  } in
+    out_newline = ignore; out_spaces = ignore ; out_indent = ignore;
+  }[@warning "-23"] in
   (* Replace formatter functions. *)
   List.iter (fun pp -> Format.pp_set_formatter_out_functions pp out_functions) pps;
   try
@@ -288,9 +291,8 @@ let parse_default parse str eos_is_error =
       | Syntaxerr.Ill_formed_ast (loc, s) ->
           Error ([mkloc loc],
                  Printf.sprintf "Error: broken invariant in parsetree: %s" s)
-      | Syntaxerr.Invalid_package_type (loc, s) ->
-          Error ([mkloc loc],
-                 Printf.sprintf "Invalid package type: %s" s)
+      | Syntaxerr.Invalid_package_type (loc, err) ->
+          Error ([mkloc loc], UTop_compat.invalid_package_error_to_string err)
 #if OCAML_VERSION >= (5, 0, 0)
       | Syntaxerr.Removed_string_set loc ->
           Error ([mkloc loc],
@@ -306,7 +308,7 @@ let parse_default parse str eos_is_error =
     | exn ->
         Error ([], "Unknown parsing error (please report it to the utop project): " ^ Printexc.to_string exn)
 
-let parse_toplevel_phrase_default = parse_default Parse.toplevel_phrase
+let parse_toplevel_phrase_default = fun str -> parse_default !Toploop.parse_toplevel_phrase str
 let parse_toplevel_phrase = ref parse_toplevel_phrase_default
 
 let parse_use_file_default = parse_default Parse.use_file
@@ -351,18 +353,18 @@ let check_phrase phrase =
         (* Construct "let _ () = let module _ = struct <items> end in ()" in order to test
            the typing and compilation of [items] without evaluating them. *)
         let unit =
-          let (%.) a b = Longident.Ldot (a, b) in
-          with_loc loc (Lident "Stdlib" %. "Unit" %. "()")
+          with_loc loc (UTop_compat.longident_of_list ["Stdlib"; "Unit"; "()"])
         in
         let top_def =
           let open Ast_helper in
           with_default_loc loc
             (fun () ->
-               Str.eval
-                 (Exp.fun_ Nolabel None (Pat.construct unit None)
-                   (Exp.letmodule (with_loc loc (Some "_"))
+              let punit = (Pat.construct unit None) in
+              let body = (Exp.letmodule ~loc:loc
+                      (with_loc loc (Some "_"))
                       (Mod.structure (item :: items))
-                      (Exp.construct unit None))))
+                      (Exp.construct unit None)) in
+              Str.eval (UTop_compat.Exp.fun_ ~loc punit body))
         in
         let check_phrase = Ptop_def [top_def] in
         try
@@ -828,7 +830,8 @@ let () =
    | Compiler-libs re-exports                                        |
    +-----------------------------------------------------------------+ *)
 
-let get_load_path () = Load_path.get_paths ()
+let get_load_path = UTop_compat.get_load_path
+
 let set_load_path = UTop_compat.set_load_path
 
 module Private = struct
